import argparse
import os.path
import subprocess
import sys

from all_repos import cli
from all_repos import color
from all_repos.config import load_config


class GrepError(ValueError):
    pass


def grep_result(config, repo, args):
    path = os.path.join(config.output_dir, repo)
    ret = subprocess.run(
        ('git', '-C', path, 'grep', *args), stdout=subprocess.PIPE,
    )
    return path, ret.returncode, ret.stdout


def grep(config, grep_args):
    repos = config.get_cloned_repos()
    ret = {}
    for repo in repos:
        repo, returncode, stdout = grep_result(config, repo, grep_args)
        if returncode == 0:
            ret[repo] = stdout
        elif returncode != 1:
            raise GrepError(returncode)
    return ret


def repos_matching(config, grep_args):
    return set(grep(config, ('--quiet', *grep_args)))


def repos_matching_cli(config, grep_args):
    try:
        matching = repos_matching(config, grep_args)
    except GrepError as e:
        return e.args[0]
    for repo in sorted(matching):
        print(repo)
    return int(not matching)


def grep_cli(config, grep_args, *, output_paths, use_color):
    sep = os.sep.encode() if output_paths else b':'
    if use_color:
        grep_args = ('--color=always', *grep_args)
    try:
        matching = grep(config, grep_args)
    except GrepError as e:
        return e.args[0]
    for repo, stdout in sorted(matching.items()):
        repo_b = repo.encode()
        for line in stdout.splitlines():
            sys.stdout.buffer.write(
                color.fmtb(repo_b, color.BLUE_B, use_color=use_color) +
                color.fmtb(sep, color.TURQUOISE, use_color=use_color) +
                line + b'\n',
            )
            sys.stdout.buffer.flush()
    return int(not matching)


def main(argv=None):
    parser = argparse.ArgumentParser(
        description='Similar to a distributed `git grep ...`.',
        usage='%(prog)s [options] [GIT_GREP_OPTIONS]',
    )
    cli.add_common_args(parser)
    cli.add_repos_with_matches_arg(parser)
    cli.add_output_paths_arg(parser)
    args, rest = parser.parse_known_args(argv)

    config = load_config(args.config_filename)
    if args.repos_with_matches:
        return repos_matching_cli(config, rest)
    else:
        return grep_cli(
            config, rest, output_paths=args.output_paths, use_color=args.color,
        )


if __name__ == '__main__':
    exit(main())



import argparse
import os.path
import sys

import yaml
from pre_commit.constants import CONFIG_FILE

from all_repos import autofix_lib
from all_repos.autofix.pre_commit_autoupdate import check_fix
from all_repos.autofix.pre_commit_autoupdate import find_repos as _find_repos
from all_repos.autofix.pre_commit_autoupdate import tmp_pre_commit_home


def apply_fix():
    autofix_lib.run(sys.executable, '-m', 'pre_commit', 'migrate-config')


def _has_legacy_config(repo_dir):
    with open(os.path.join(repo_dir, CONFIG_FILE)) as f:
        contents = yaml.load(f.read())
    return isinstance(contents, list)


def find_repos(config):
    return {repo for repo in _find_repos(config) if _has_legacy_config(repo)}


def main(argv=None):
    parser = argparse.ArgumentParser()
    autofix_lib.add_fixer_args(parser)
    args = parser.parse_args(argv)

    autofix_lib.assert_importable('pre_commit', install='pre-commit')
    # pre-commit 1.0.0: introduces migrate-config
    # pre-commit 1.0.1: exit code fix
    # pre-commit 1.7.0: sha -> rev
    autofix_lib.require_version_gte('pre-commit', '1.7.0')

    repos, config, commit, autofix_settings = autofix_lib.from_cli(
        args,
        find_repos=find_repos,
        msg='Ran pre-commit migrate-config.',
        branch_name='pre-commit-migrate-config',
    )

    with tmp_pre_commit_home():
        autofix_lib.fix(
            repos,
            apply_fix=apply_fix,
            check_fix=check_fix,
            config=config,
            commit=commit,
            autofix_settings=autofix_settings,
        )


if __name__ == '__main__':
    exit(main())



import collections
import json
from typing import Dict


Settings = collections.namedtuple('Settings', ('filename',))


def list_repos(settings: Settings) -> Dict[str, str]:
    with open(settings.filename) as f:
        return json.load(f)



import collections
import json
import os
import re


class Config(collections.namedtuple(
        'Config',
        (
            'output_dir', 'include', 'exclude',
            'list_repos', 'source_settings',
            'push', 'push_settings',
        ),
)):
    __slots__ = ()

    def _path(self, *paths):
        return os.path.abspath(os.path.join(self.output_dir, *paths))

    @property
    def repos_path(self):
        return self._path('repos.json')

    @property
    def repos_filtered_path(self):
        return self._path('repos_filtered.json')

    def get_cloned_repos(self):
        with open(self.repos_filtered_path) as f:
            return json.load(f)


def _check_permissions(filename: str) -> None:
    mode = os.stat(filename).st_mode & 0o777
    if mode != 0o600:
        raise SystemExit(
            f'{filename} has too-permissive permissions, Expected 0o600, '
            f'got 0o{mode:o}',
        )


def load_config(filename: str) -> Config:
    _check_permissions(filename)
    with open(filename) as f:
        contents = json.load(f)

    output_dir = os.path.join(filename, '..', contents['output_dir'])
    output_dir = os.path.normpath(output_dir)
    source_module = __import__(contents['source'], fromlist=['__trash'])
    source_settings = source_module.Settings(**contents['source_settings'])
    push_module = __import__(contents['push'], fromlist=['__trash'])
    push_settings = push_module.Settings(**contents['push_settings'])
    include = re.compile(contents.get('include', ''))
    exclude = re.compile(contents.get('exclude', '^$'))
    return Config(
        output_dir=output_dir, include=include, exclude=exclude,
        list_repos=source_module.list_repos, source_settings=source_settings,
        push=push_module.push, push_settings=push_settings,
    )



import collections
import json
import urllib.request
from typing import Dict
from typing import List


Settings = collections.namedtuple(
    'Settings', ('api_key', 'username', 'collaborator', 'forks', 'private'),
)
Settings.__new__.__defaults__ = (False, False, False)


def _parse_link_header(lnk):
    if lnk is None:
        return {}

    ret = {}
    parts = lnk.split(',')
    for part in parts:
        link, _, rel = part.partition(';')
        link, rel = link.strip(), rel.strip()
        assert link.startswith('<') and link.endswith('>'), link
        assert rel.startswith('rel="') and rel.endswith('"'), rel
        link, rel = link[1:-1], rel[len('rel="'):-1]
        ret[rel] = link
    return ret


def _req(*args, **kwargs):
    resp = urllib.request.urlopen(urllib.request.Request(*args, **kwargs))
    return json.loads(resp.read()), _parse_link_header(resp.headers['link'])


def _get_all(url: str, **kwargs) -> List[dict]:
    ret = []
    resp, links = _req(url, **kwargs)
    ret.extend(resp)
    while 'next' in links:
        resp, links = _req(links['next'], **kwargs)
        ret.extend(resp)
    return ret


def list_repos(settings: Settings) -> Dict[str, str]:
    repos = _get_all(
        'https://api.github.com/user/repos?per_page=100',
        headers={'Authorization': f'token {settings.api_key}'},
    )
    return {
        repo['full_name']: 'git@github.com:{}'.format(repo['full_name'])
        for repo in repos
        if (
            (settings.forks or not repo['fork']) and
            (settings.private or not repo['private']) and
            (settings.collaborator or repo['permissions']['admin'])
        )
    }


def better_repr(obj):
    if isinstance(obj, list):
        return [better_repr(o) for o in obj]
    elif isinstance(obj, dict):
        return {
            k: better_repr(v) for k, v in obj.items() if not k.endswith('url')
        }
    else:
        return obj



import collections
import json
import subprocess
from typing import Dict
from typing import List


class Settings(collections.namedtuple(
        'Settings', ('username', 'hostname', 'mirror_path'),
)):

    __slots__ = ()

    def clone_url(self, repo_name):
        return (
            self.mirror_path or
            f'{self.username}@{self.hostname}:{{repo_name}}'
        ).format(repo_name=repo_name)


Settings.__new__.__defaults__ = (None,)


def _repo_names_from_source(settings: Settings) -> List[str]:
    info = subprocess.check_output(
        ('ssh', f'{settings.username}@{settings.hostname}', 'info', '-json'),
    )
    info = json.loads(info.decode('UTF-8'))
    return set(info['repos'])


def list_repos(settings: Settings) -> Dict[str, str]:
    return {
        # Repo names have ".git" appended to avoid naming conflicts between
        # repos and directories in the gitolite hierarchy (a path could
        # otherwise be both).
        f'{repo_name}.git': settings.clone_url(repo_name)
        for repo_name in _repo_names_from_source(settings)
    }



import collections
import contextlib
import functools
import os
import pipes
import subprocess
import tempfile
import traceback

import pkg_resources

from all_repos import cli
from all_repos import color
from all_repos import git
from all_repos import mapper
from all_repos.config import Config
from all_repos.config import load_config


def add_fixer_args(parser):
    cli.add_common_args(parser)

    mutex = parser.add_mutually_exclusive_group()
    mutex.add_argument(
        '--dry-run', action='store_true',
        help='show what would happen but do not push.',
    )
    mutex.add_argument(
        '-i', '--interactive', action='store_true',
        help='interactively approve / deny fixes.',
    )
    cli.add_jobs_arg(mutex, default=1)

    parser.add_argument(
        '--limit', type=int, default=None,
        help='maximum number of repos to process (default: unlimited).',
    )
    parser.add_argument(
        '--author',
        help=(
            'override commit author.  '
            'This is passed directly to `git commit`.  '
            "An example: `--author='Herp Derp <herp.derp@umich.edu>'`."
        ),
    )
    parser.add_argument(
        '--repos', nargs='*',
        help=(
            'run against specific repositories instead.  This is especially '
            'useful with `xargs autofixer ... --repos`.  This can be used to '
            'specify repositories which are not managed by `all-repos`.'
        ),
    )


class Commit(collections.namedtuple(
        'Commit', ('msg', 'branch_name', 'author'),
)):
    __slots__ = ()

    @classmethod
    def from_cli(cls, args, *, msg, branch_name):
        return cls(msg=msg, branch_name=branch_name, author=args.author)


class AutofixSettings(collections.namedtuple(
        'AutofixSettings',
        ('jobs', 'color', 'limit', 'dry_run', 'interactive'),
)):
    __slots__ = ()

    @classmethod
    def from_cli(cls, args):
        return cls(
            jobs=args.jobs, color=args.color, limit=args.limit,
            dry_run=args.dry_run, interactive=args.interactive,
        )


def filter_repos(config, cli_repos, find_repos):
    if cli_repos is not None:
        return cli_repos
    else:
        return find_repos(config)


def from_cli(args, *, find_repos, msg, branch_name):
    config = load_config(args.config_filename)
    return (
        filter_repos(config, args.repos, find_repos),
        config,
        Commit.from_cli(args, msg=msg, branch_name=branch_name),
        AutofixSettings.from_cli(args),
    )


def run(*cmd, **kwargs):
    cmdstr = ' '.join(pipes.quote(arg) for arg in cmd)
    print(f'$ {cmdstr}', flush=True)
    kwargs.setdefault('check', True)
    return subprocess.run(cmd, **kwargs)


@contextlib.contextmanager
def cwd(path):
    pwd = os.getcwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(pwd)


def assert_importable(module: str, *, install: str) -> None:
    try:
        __import__(module)
    except ImportError:
        raise SystemExit(
            f'This tool requires the `{module}` module to be installed.\n'
            f'Try installing it via `pip install {install}`.',
        )


def require_version_gte(pkg_name: str, version: str) -> None:
    pkg = pkg_resources.get_distribution(pkg_name)
    pkg_version = pkg_resources.parse_version(pkg.version)
    target_version = pkg_resources.parse_version(version)
    if pkg_version < target_version:
        raise SystemExit(
            f'This tool requires the `{pkg_name}` package is at least version '
            f'{version}.  '
            f'The currently installed version is {pkg.version}.\n\n'
            f'Try `pip install --upgrade {pkg_name}`',
        )


@contextlib.contextmanager
def repo_context(repo, *, use_color):
    print(color.fmt(f'***{repo}', color.TURQUOISE_H, use_color=use_color))
    try:
        remote = git.remote(repo)
        with tempfile.TemporaryDirectory() as tmpdir:
            run('git', 'clone', '--quiet', repo, tmpdir)
            with cwd(tmpdir):
                run('git', 'remote', 'set-url', 'origin', remote)
                run('git', 'fetch', '--prune', '--quiet')
                yield
    except Exception:
        print(color.fmt(f'***Errored', color.RED_H, use_color=use_color))
        traceback.print_exc()


def _interactive_check(*, use_color):
    def _quit():
        print('Goodbye!')
        raise SystemExit()

    while True:
        try:
            s = input(color.fmt(
                '***Looks good [y,n,s,q,?]? ',
                color.BLUE_B, use_color=use_color,
            ))
        except (EOFError, KeyboardInterrupt):
            _quit()

        s = s.strip().lower()
        if s in {'y', 'yes'}:
            return True
        elif s in {'n', 'no'}:
            return False
        elif s in {'s', 'shell'}:
            print('Opening an interactive shell, type `exit` to continue.')
            print('Any modifications will be committed.')
            subprocess.call(os.environ.get('SHELL', 'bash'))
        elif s in {'q', 'quit'}:
            _quit()
        else:
            if s not in {'?', 'help'}:
                print(color.fmt(
                    f'Unexpected input: {s}', color.RED, use_color=use_color,
                ))
            print('y (yes): yes it looks good, commit and continue.')
            print('n (no): no, do not commit this repository.')
            print('s (shell): open an interactive shell in the repo.')
            print('q (quit, ^C): early exit from the autofixer.')
            print('? (help): show this help message.')


def _fix_inner(repo, apply_fix, check_fix, config, commit, autofix_settings):
    with repo_context(repo, use_color=autofix_settings.color):
        branch_name = f'all-repos_autofix_{commit.branch_name}'
        run('git', 'checkout', '--quiet', 'origin/master', '-b', branch_name)

        apply_fix()

        diff = run('git', 'diff', 'origin/master', '--exit-code', check=False)
        if not diff.returncode:
            return

        check_fix()

        if (
                autofix_settings.interactive and
                not _interactive_check(use_color=autofix_settings.color)
        ):
            return

        commit_message = (
            f'{commit.msg}\n\n'
            f'Committed via https://github.com/asottile/all-repos'
        )
        commit_cmd = ('git', 'commit', '--quiet', '-a', '-m', commit_message)
        if commit.author:
            commit_cmd += ('--author', commit.author)

        run(*commit_cmd)

        if autofix_settings.dry_run:
            return

        config.push(config.push_settings, branch_name)


def _noop_check_fix():
    """A lambda is not pickleable, this must be a module-level function"""


def fix(
        repos, *,
        apply_fix,
        check_fix=_noop_check_fix,
        config: Config,
        commit: Commit,
        autofix_settings: AutofixSettings,
):
    assert not autofix_settings.interactive or autofix_settings.jobs == 1
    repos = tuple(repos)[:autofix_settings.limit]
    func = functools.partial(
        _fix_inner,
        apply_fix=apply_fix, check_fix=check_fix,
        config=config, commit=commit, autofix_settings=autofix_settings,
    )
    with mapper.process_mapper(autofix_settings.jobs) as do_map:
        mapper.exhaust(do_map(func, repos))



import argparse
import contextlib
import os
import sys
import tempfile

from all_repos import autofix_lib
from all_repos.grep import repos_matching


@contextlib.contextmanager
def tmp_pre_commit_home(*, _absent=object()):
    """During lots of autoupdates, many repositories will be cloned into the
    pre-commit directory.  This prevents leaving many MB/GB of repositories
    behind due to this autofixer.  This context creates a temporary directory
    so these many repositories are automatically cleaned up.
    """
    before = os.environ.get('PRE_COMMIT_HOME', _absent)
    with tempfile.TemporaryDirectory() as tmpdir:
        os.environ['PRE_COMMIT_HOME'] = tmpdir
        try:
            yield
        finally:
            if before is _absent:
                os.environ.pop('PRE_COMMIT_HOME', None)
            else:
                os.environ['PRE_COMMIT_HOME'] = before


def _run_all_files(**kwargs):
    autofix_lib.run(
        sys.executable, '-m', 'pre_commit', 'run', '--all-files', **kwargs,
    )


def find_repos(config):
    return repos_matching(config, ('', '--', '.pre-commit-config.yaml'))


def apply_fix():
    autofix_lib.run(sys.executable, '-m', 'pre_commit', 'autoupdate')
    # This may return nonzero for fixes, that's ok!
    _run_all_files(check=False)


def check_fix():
    _run_all_files()


def main(argv=None):
    parser = argparse.ArgumentParser()
    autofix_lib.add_fixer_args(parser)
    args = parser.parse_args(argv)

    autofix_lib.assert_importable('pre_commit', install='pre-commit')
    # pre-commit 0.16.3: autoupdate maintains formatting better
    # pre-commit 0.17.0: race conditions in pre-commit install fixed
    # pre-commit 1.0.0: migrate_config(...) is called for autoupdate
    # pre-commit 1.0.1: exit code fix
    # pre-commit 1.7.0: sha -> rev
    autofix_lib.require_version_gte('pre-commit', '1.7.0')

    repos, config, commit, autofix_settings = autofix_lib.from_cli(
        args,
        find_repos=find_repos,
        msg='Ran pre-commit autoupdate.', branch_name='pre-commit-autoupdate',
    )

    with tmp_pre_commit_home():
        autofix_lib.fix(
            repos,
            apply_fix=apply_fix,
            check_fix=check_fix,
            config=config,
            commit=commit,
            autofix_settings=autofix_settings,
        )


if __name__ == '__main__':
    exit(main())



import argparse

from all_repos import cli
from all_repos.config import load_config


def main(argv=None):
    parser = argparse.ArgumentParser(
        description='List all cloned repository names.',
        usage='all-repos-list-repos [options]',
    )
    cli.add_common_args(parser)
    args = parser.parse_args(argv)

    config = load_config(args.config_filename)
    for repo in config.get_cloned_repos():
        print(repo)


if __name__ == '__main__':
    exit(main())



import argparse

from all_repos import cli
from all_repos.config import load_config

BASH = '''\
__all_repos__clone_opts="$(git clone --help | grep -Eo -- '--[a-zA-Z0-9-]+')"

# https://github.com/scop/bash-completion/blob/d2f14a7/bash_completion#L498
__all_repos__ltrim_colon_completions() {
    if [[ "$1" == *:* && "$COMP_WORDBREAKS" == *:* ]]; then
        # Remove colon-word prefix from COMPREPLY items
        local colon_word=${1%"${1##*:}"}
        local i=${#COMPREPLY[*]}
        while [[ $((--i)) -ge 0 ]]; do
            COMPREPLY[$i]=${COMPREPLY[$i]#"$colon_word"}
        done
    fi
}

_git_clone() {
    case "$cur" in
    --*)
        __gitcomp "$__all_repos__clone_opts"
        return
        ;;
    *)
        argc=0
        for word in "${words[@]}"; do
            case "$word" in
            git|clone|--*)
                continue
                ;;
            *)
                argc=$((argc + 1))
                ;;
            esac
        done

        if [ $argc -le 1 ]; then
            __gitcomp "$(jq --raw-output .[] "$__all_repos__repos_json")"
            __all_repos__ltrim_colon_completions "$cur"
        fi
        ;;
    esac
}
'''

ZSH = '''\
__git_remote_repositories() {
    local -a options
    options=( $(
        jq -r \
           'to_entries | map([.value, .key]|join("\\t")) | join("\\n")' \
           "$__all_repos__repos_json" |
            sed 's/:/\\\\:/; s/\\t/:/'
    ) )
    _describe 'values' options
}
'''


def main(argv=None):
    parser = argparse.ArgumentParser(
        description=(
            'Add git-clone tab completion for all-repos repositories.\n\n'
            'Add to .bash_profile:\n'
            '  `eval "$(all-repos-complete -C ~/.../all-repos.json --bash)"`'
        ),
        formatter_class=argparse.RawDescriptionHelpFormatter,
        usage='%(prog)s [options] {--bash}',
    )
    cli.add_common_args(parser)
    mutex = parser.add_mutually_exclusive_group(required=True)
    mutex.add_argument('--bash', action='store_true')
    mutex.add_argument('--zsh', action='store_true')
    args = parser.parse_args(argv)

    config = load_config(args.config_filename)

    print(f'__all_repos__repos_json={config.repos_filtered_path}')
    if args.bash:
        print(BASH)
    elif args.zsh:
        print(ZSH)
    else:
        raise NotImplementedError()


if __name__ == '__main__':
    exit(main())



import multiprocessing
import os
import sys


def jobs_type(s):
    jobs = int(s)
    if jobs <= 0:
        return multiprocessing.cpu_count()
    else:
        return jobs


def add_jobs_arg(parser, default=8):
    parser.add_argument(
        '-j', '--jobs', type=jobs_type, default=default,
        help=(
            'how many concurrent jobs will be used to complete the '
            'operation.  Specify 0 or -1 to match the number of cpus '
            '(default `%(default)s`).'
        ),
    )


COLOR_CHOICES = ('auto', 'always', 'never')


def use_color(setting):
    if setting not in COLOR_CHOICES:
        raise ValueError(setting)
    return (
        setting == 'always' or
        (setting == 'auto' and sys.stdout.isatty())
    )


def add_common_args(parser):
    parser.add_argument(
        '-C', '--config-filename',
        default=os.getenv('ALL_REPOS_CONFIG_FILENAME') or 'all-repos.json',
        help='use a non-default config file (default `%(default)s`).',
    )
    parser.add_argument(
        '--color', default='auto', type=use_color,
        metavar='{' + ','.join(COLOR_CHOICES) + '}',
        help='use color in output (default `%(default)s`).',
    )


def add_repos_with_matches_arg(parser):
    parser.add_argument(
        '--repos-with-matches', action='store_true',
        help='only print repositories with matches.',
    )


def add_output_paths_arg(parser):
    parser.add_argument(
        '--output-paths', action='store_true',
        help=(
            f'Use `{os.sep}` as a separator instead of `:` in outputs (often '
            f'helpful for scripting).'
        ),
    )



import collections
import json
import subprocess
import urllib.request

from all_repos import autofix_lib
from all_repos import git


Settings = collections.namedtuple('Settings', ('api_key', 'username', 'fork'))
Settings.__new__.__defaults__ = (False,)


def push(settings: Settings, branch_name: str) -> None:
    headers = {'Authorization': f'token {settings.api_key}'}

    remote_url = git.remote('.')
    _, _, repo_slug = remote_url.rpartition(':')

    if settings.fork:
        resp = urllib.request.urlopen(urllib.request.Request(
            f'https://api.github.com/repos/{repo_slug}/forks',
            headers=headers, method='POST',
        ))
        new_slug = json.loads(resp.read())['full_name']
        new_remote = remote_url.replace(repo_slug, new_slug)
        autofix_lib.run('git', 'remote', 'add', 'fork', new_remote)
        remote = 'fork'
        head = f'{settings.username}:{branch_name}'
    else:
        remote = 'origin'
        head = branch_name

    autofix_lib.run('git', 'push', remote, f'HEAD:{branch_name}', '--quiet')

    title = subprocess.check_output(('git', 'log', '-1', '--format=%s'))
    body = subprocess.check_output(('git', 'log', '-1', '--format=%b'))

    data = json.dumps({
        'title': title.decode().strip(),
        'body': body.decode().strip(),
        'base': 'master',
        'head': head,
    }).encode()

    resp = urllib.request.urlopen(urllib.request.Request(
        f'https://api.github.com/repos/{repo_slug}/pulls',
        data=data, headers=headers, method='POST',
    ))

    url = json.loads(resp.read())['html_url']
    print(f'Pull request created at {url}')



import argparse
import os.path
import re
import subprocess
import sys

from all_repos import cli
from all_repos import color
from all_repos.config import load_config
from all_repos.util import zsplit


def ls_files(config, repo):
    path = os.path.join(config.output_dir, repo)
    ret = subprocess.run(
        ('git', '-C', path, 'ls-files', '-z'),
        stdout=subprocess.PIPE, check=True,
    )
    return path, zsplit(ret.stdout)


def find_files(config, pattern):
    regex = re.compile(pattern.encode())
    repos = config.get_cloned_repos()
    ret = {}
    for repo in repos:
        repo, filenames = ls_files(config, repo)
        matched = [f for f in filenames if regex.search(f)]
        if matched:
            ret[repo] = matched
    return ret


def find_files_repos_cli(config, pattern, *, use_color):
    repo_files = find_files(config, pattern)
    for repo in repo_files:
        print(repo)
    return not repo_files


def find_files_cli(config, pattern, *, output_paths, use_color):
    sep = os.sep.encode() if output_paths else b':'
    repo_files = find_files(config, pattern)
    for repo, matching in repo_files.items():
        for filename in matching:
            sys.stdout.buffer.write(
                color.fmtb(repo.encode(), color.BLUE_B, use_color=use_color) +
                color.fmtb(sep, color.TURQUOISE, use_color=use_color) +
                filename + b'\n',
            )
    return not repo_files


def main(argv=None):
    parser = argparse.ArgumentParser(
        description=(
            'Similar to a distributed `git ls-files | grep -P PATTERN`.'
        ),
        usage='%(prog)s [options] PATTERN',
    )
    cli.add_common_args(parser)
    cli.add_repos_with_matches_arg(parser)
    cli.add_output_paths_arg(parser)
    parser.add_argument('pattern', help='the python regex to match.')
    args = parser.parse_args(argv)

    config = load_config(args.config_filename)
    if args.repos_with_matches:
        return find_files_repos_cli(config, args.pattern, use_color=args.color)
    else:
        return find_files_cli(
            config, args.pattern,
            output_paths=args.output_paths, use_color=args.color,
        )


if __name__ == '__main__':
    exit(main())



import argparse
import functools
import os.path
import shlex
import subprocess

from identify.identify import tags_from_path

from all_repos import autofix_lib
from all_repos.util import zsplit


def find_repos(config, *, ls_files_cmd):
    for repo in config.get_cloned_repos():
        repo_dir = os.path.join(config.output_dir, repo)
        if subprocess.run(
            ('git', '-C', repo_dir, *ls_files_cmd[1:]),
            check=True, stdout=subprocess.PIPE,
        ).stdout:
            yield repo_dir


def apply_fix(*, ls_files_cmd, sed_cmd):
    filenames = zsplit(subprocess.check_output(ls_files_cmd))
    filenames = [f.decode() for f in filenames]
    filenames = [f for f in filenames if tags_from_path(f) & {'file', 'text'}]
    autofix_lib.run(*sed_cmd, *filenames)


def _quote_cmd(cmd):
    return ' '.join(shlex.quote(arg) for arg in cmd)


def main(argv=None):
    parser = argparse.ArgumentParser(
        description=(
            'Similar to a distributed '
            '`git ls-files -z -- FILENAMES | xargs -0 sed -i EXPRESSION`.'
        ),
        usage='%(prog)s [options] EXPRESSION FILENAMES',
    )
    autofix_lib.add_fixer_args(parser)
    parser.add_argument(
        '-r', '--regexp-extended',
        action='store_true',
        help='use extended regular expressions in the script.',
    )
    parser.add_argument(
        '--branch-name', default='all-repos-sed',
        help='override the autofixer branch name (default `%(default)s`).',
    )
    parser.add_argument(
        '--commit-msg',
        help=(
            'override the autofixer commit message.  (default '
            '`git ls-files -z -- FILENAMES | xargs -0 sed -i ... EXPRESSION`).'
        ),
    )
    parser.add_argument(
        'expression', help='sed program. For example: `s/hi/hello/g`.',
    )
    parser.add_argument(
        'filenames', help='filenames glob (passed to `git ls-files`).',
    )
    args = parser.parse_args(argv)

    dash_r = ('-r',) if args.regexp_extended else ()
    sed_cmd = ('sed', '-i', *dash_r, args.expression)
    ls_files_cmd = ('git', 'ls-files', '-z', '--', args.filenames)

    msg = f'{_quote_cmd(ls_files_cmd)} | xargs -0 {_quote_cmd(sed_cmd)}'
    msg = args.commit_msg or msg

    repos, config, commit, autofix_settings = autofix_lib.from_cli(
        args,
        find_repos=functools.partial(find_repos, ls_files_cmd=ls_files_cmd),
        msg=msg, branch_name=args.branch_name,
    )

    autofix_lib.fix(
        repos,
        apply_fix=functools.partial(
            apply_fix, ls_files_cmd=ls_files_cmd, sed_cmd=sed_cmd,
        ),
        config=config, commit=commit, autofix_settings=autofix_settings,
    )


if __name__ == '__main__':
    exit(main())



import concurrent.futures
import contextlib


def exhaust(gen):
    for _ in gen:
        pass


@contextlib.contextmanager
def _in_process():
    yield map


@contextlib.contextmanager
def _threads(jobs):
    with concurrent.futures.ThreadPoolExecutor(jobs) as ex:
        yield ex.map


def thread_mapper(jobs):
    if jobs == 1:
        return _in_process()
    else:
        return _threads(jobs)


@contextlib.contextmanager
def _processes(jobs):
    with concurrent.futures.ProcessPoolExecutor(jobs) as ex:
        yield ex.map


def process_mapper(jobs):
    if jobs == 1:
        return _in_process()
    else:
        return _processes(jobs)



import collections

from all_repos import autofix_lib


Settings = collections.namedtuple('Settings', ())


def push(settings: Settings, branch_name: str) -> None:
    autofix_lib.run('git', 'checkout', 'master', '--quiet')
    autofix_lib.run('git', 'pull', '--quiet')
    autofix_lib.run(
        'git', 'merge', '--no-edit', '--no-ff', branch_name, '--quiet',
    )
    autofix_lib.run('git', 'push', 'origin', 'HEAD', '--quiet')



BLUE_B = '\033[1;34m'
RED = '\033[31m'
RED_H = '\033[41m'
TURQUOISE = '\033[36m'
TURQUOISE_H = '\033[46;30m'
NORMAL = '\033[m'


def fmt(text, color, *, use_color):
    if use_color:
        return f'{color}{text}{NORMAL}'
    else:
        return text


def fmtb(bs, color, *, use_color):
    if use_color:
        return b'%s%s%s' % (color.encode(), bs, NORMAL.encode())
    else:
        return bs



import argparse
import json
import os.path
import shutil
import subprocess
from typing import Dict

from all_repos import cli
from all_repos import mapper
from all_repos.config import load_config


def _git_remote(path: str) -> str:
    return subprocess.check_output((
        'git', '-C', path, 'config', 'remote.origin.url',
    )).decode().strip()


def _get_current_state_helper(path):
    if not os.path.exists(path):
        return

    pths = []
    seen_git = False
    for direntry in os.scandir(path):
        if direntry.name == '.git':
            seen_git = True
        elif direntry.is_dir():  # pragma: no branch (defensive)
            pths.append(direntry)
    if seen_git:
        yield path, _git_remote(path)
    else:
        for pth in pths:
            yield from _get_current_state_helper(pth)


def _get_current_state(path: str) -> Dict[str, str]:
    return {
        os.path.relpath(k, path): v for k, v in _get_current_state_helper(path)
    }


def _remove(dest: str, path: str) -> None:
    print(f'Removing {path}')
    shutil.rmtree(os.path.join(dest, path))
    # Remove any empty directories
    path = os.path.dirname(path)
    while path and not os.listdir(os.path.join(dest, path)):
        os.rmdir(os.path.join(dest, path))
        path = os.path.dirname(path)


def _init(dest: str, path: str, remote: str) -> None:
    print(f'Initializing {path}')
    path = os.path.join(dest, path)
    os.makedirs(path, exist_ok=True)
    subprocess.check_call(('git', 'init', path))
    subprocess.check_output((
        'git', '-C', path, 'remote', 'add',
        'origin', remote, '--track', 'master',
    ))


def _fetch_reset(path: str) -> None:
    try:
        subprocess.check_call(('git', '-C', path, 'fetch'))
        subprocess.check_call((
            'git', '-C', path, 'reset', '--hard', 'origin/master',
        ))
    except subprocess.CalledProcessError:
        # TODO: color / tty
        print(f'Error fetching {path}')


def main(argv=None):
    parser = argparse.ArgumentParser(
        description='Clone all the repositories into the `output_dir`.',
        usage='%(prog)s [options]',
    )
    cli.add_common_args(parser)
    cli.add_jobs_arg(parser)
    args = parser.parse_args(argv)

    config = load_config(args.config_filename)

    repos = config.list_repos(config.source_settings)
    repos_filtered = {
        k: v for k, v in repos.items()
        if config.include.search(k) and not config.exclude.search(k)
    }

    # If the previous `repos.json` / `repos_filtered.json` files exist
    # remove them.
    for path in (config.repos_path, config.repos_filtered_path):
        if os.path.exists(path):
            os.remove(path)

    current_repos = set(_get_current_state(config.output_dir).items())
    filtered_repos = set(repos_filtered.items())

    # Remove old no longer cloned repositories
    for path, _ in current_repos - filtered_repos:
        _remove(config.output_dir, path)

    for path, remote in filtered_repos - current_repos:
        _init(config.output_dir, path, remote)

    todo = [os.path.join(config.output_dir, p) for p in repos_filtered]
    with mapper.thread_mapper(args.jobs) as do_map:
        mapper.exhaust(do_map(_fetch_reset, todo))

    # write these last
    os.makedirs(config.output_dir, exist_ok=True)
    with open(config.repos_path, 'w') as f:
        f.write(json.dumps(repos))
    with open(config.repos_filtered_path, 'w') as f:
        f.write(json.dumps(repos_filtered))


if __name__ == '__main__':
    exit(main())



from setuptools import find_packages
from setuptools import setup

setup(
    name='all_repos',
    description='Clone all your repositories and apply sweeping changes.',
    url='https://github.com/asottile/all-repos',
    version='1.4.0',
    author='Anthony Sottile',
    author_email='asottile@umich.edu',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=['identify'],
    packages=find_packages(exclude=('tests*', 'testing*')),
    entry_points={
        'console_scripts': [
            'all-repos-clone=all_repos.clone:main',
            'all-repos-complete=all_repos.complete:main',
            'all-repos-find-files=all_repos.find_files:main',
            'all-repos-grep=all_repos.grep:main',
            'all-repos-list-repos=all_repos.list_repos:main',
            'all-repos-sed=all_repos.sed:main',
        ],
    },
)



import argparse
import os.path

from all_repos import autofix_lib
from all_repos.grep import repos_matching


APPVEYOR = 'appveyor.yml'
TRAVIS = '.travis.yml'


def find_repos(config):
    return (
        repos_matching(config, ('$HOME/.pre-commit', '--', TRAVIS)) |
        repos_matching(config, (r'%USERPROFILE%\\.pre-commit', '--', APPVEYOR))
    )


def _replace_if_exists(filename, s1, s2):
    if os.path.exists(filename):
        with open(filename) as f:
            contents = f.read()
        contents = contents.replace(s1, s2)
        with open(filename, 'w') as f:
            f.write(contents)


def apply_fix():
    _replace_if_exists(TRAVIS, '$HOME/.pre-commit', '$HOME/.cache/pre-commit')
    _replace_if_exists(
        APPVEYOR,
        r'%USERPROFILE%\.pre-commit', r'%USERPROFILE%\.cache\pre-commit',
    )


def main(argv=None):
    parser = argparse.ArgumentParser()
    autofix_lib.add_fixer_args(parser)
    args = parser.parse_args(argv)

    repos, config, commit, autofix_settings = autofix_lib.from_cli(
        args,
        find_repos=find_repos,
        msg='Update pre-commit cache directory.',
        branch_name='pre-commit-cache-dir',
    )

    autofix_lib.fix(
        repos,
        apply_fix=apply_fix,
        config=config,
        commit=commit,
        autofix_settings=autofix_settings,
    )


if __name__ == '__main__':
    exit(main())
